package meerkat.Registry;

import com.google.protobuf.InvalidProtocolBufferException;
import meerkat.VoterRegistryMessage;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.BulletinBoardAPI.FilterType;
import meerkat.protobuf.BulletinBoardAPI.MessageFilter;
import meerkat.protobuf.BulletinBoardAPI.MessageFilterList;

import java.text.ParseException;
import java.util.*;

/**
 * TODO: add logging to this utils
 */

/**
 * Created by Vladimir Eliezer Tokarev  on 1/15/2016.
 * adds extra functionality to Messages collections
 */
public abstract class CollectionMessagesUtils {

    public static class EmptyListException extends Exception {
        public EmptyListException(String message) {
            super(message);
        }
    }

    /**
     * Converts lost of UnsignedBulletinBoardMessage to VoterRegistryMessages
     * @param messages list<VoterRegistryMessage>
     * @return List<VoterRegistryMessage>
     */
   public static List<VoterRegistryMessage> ConvertToVoterRegistryMessages(List<BulletinBoardMessage> messages){
       List<VoterRegistryMessage> voterMessages = new ArrayList<>();
       for (int i = 0 ; i < messages.size() ; i++)
       {
           voterMessages.add(new VoterRegistryMessage(messages.get(i)));
       }
       return voterMessages;
   }

    /**
     * Gets map of GroupId to basicMessage, where the basicMessages are the last actions for those groups
     * @param messages List<VoterRegistryMessage>
     * @return     Map{String:VoterRegistryMessage}
     * @throws ParseException
     */
    public static Map<String, VoterRegistryMessage> GetLatestGroupsActions(List<VoterRegistryMessage> messages)
            throws ParseException, InvalidProtocolBufferException {
        Map<String, VoterRegistryMessage> groupIdToMessage = new HashMap<>();

        // iterate trough all the messages and put into the map the last updated groups actions
        for (int i = 0 ; i < messages.size() ; i++) {
            VoterRegistryMessage message = messages.get(i);
            String groupId = message.GetWantedTagFromBasicMessage(RegistryTags.GROUP_ID_TAG);
            VoterRegistryMessage temp = groupIdToMessage.get(groupId);

            if (temp != null && temp != message) {
                if (temp.GetCreationTime() < message.GetCreationTime()) {
                    groupIdToMessage.put(groupId, message);
                }
            }
            groupIdToMessage.put(groupId, message);
        }
        return groupIdToMessage;
    }

    /**
     * Gets list of groups ids of the basicMessages that carried the adding to group tag
     * @param groupIdToMessage Map<String, VoterRegistryMessage>
     * @return List<String>
     */
     public static List<String> GetListOfGroupIds(Map<String, VoterRegistryMessage>  groupIdToMessage) {
         List<String> groupsIds = new ArrayList<>();
         Iterator entries = groupIdToMessage.entrySet().iterator();
         while (entries.hasNext()) {
             Map.Entry tuple = (Map.Entry) entries.next();
             VoterRegistryMessage message = (VoterRegistryMessage) tuple.getValue();
             String groupId = message.GetWantedTagFromBasicMessage(RegistryTags.GROUP_ID_TAG);
             groupsIds.add(groupId);
         }
         return groupsIds;
     }

    /**
     * Gets the message with the latest timestamp from messages
     * @param messages  List<VoterRegistryMessage>
     * @return VoterRegistryMessage
     * @throws ParseException
     * @throws EmptyListException
     */
    public static VoterRegistryMessage GetLatestMessage(List<VoterRegistryMessage> messages)
            throws ParseException, EmptyListException, InvalidProtocolBufferException {
        System.out.print("1");
        if (messages.size() == 0 ){
            throw new EmptyListException("The list of messages passed to GetLatestMessage is empty.");
        }
        VoterRegistryMessage LatestMessage = messages.get(0);
        for (int i = 0 ; i < messages.size() ; i++) {
            VoterRegistryMessage message = messages.get(i);
            if (message.GetCreationTime() < LatestMessage.GetCreationTime()) {
                LatestMessage = message;
            }
        }
        return LatestMessage;
    }

    /**
     * Gets messages that have the wanted id tag and have or the ADD_TO_GROUP_TAG or REMOVE_FROM_GROUP_TAG
     *
     * @param tags the tags based on which the messages will be filtered
     * @return MessageFilterList.
     */
    public static MessageFilterList GenerateFiltersFromTags(List<String> tags) {
        MessageFilterList.Builder filters = MessageFilterList.newBuilder();

        if (tags.isEmpty()){
            return filters.build();
        }

        for (int i = 0 ;i < tags.size() ; i++) {
            String tag = tags.get(i);
            MessageFilter.Builder filter = MessageFilter.newBuilder().setTag(tag).setType(FilterType.TAG);
            filters.addFilter(filter);
        }
        return filters.build();
    }
}
